#include "Song.h"

Song::Song(sf::SoundBuffer* buffer):
    fftResult(fftSamples, FFT_SAMPLES),
    sound(*buffer),
    samples(buffer->getSamples()),
    sampleRate(buffer->getSampleRate()),
    sampleCount(buffer->getSampleCount()),
    lastSampleOffset(0),
    maxAmplitude(0),
    fftSampleIndex(0),
    baseVolume(100.f), /// 1-100
    volume(1.f), /// 0-1
    loop(false)
{
    /// 44100 samples per second.
    /// 140 bpm
    /// 60 / bpm = beats per second
    /// samples per second / beats per second = samples per beat
    /// 44100 / (60 / 140) = 18900
    /// 18900 samples per beat
}

Song::~Song()
{
    //dtor
}

void Song::play()
{
    sound.play();
    sound.setVolume(baseVolume);
}


void Song::stop()
{
    sound.stop();
}

void Song::skip()
{
    sf::Time currOffset = sound.getPlayingOffset();

    sound.setPlayingOffset(sf::seconds(currOffset.asSeconds() + 2));
    printf("Skipped to %d\n", (int)(currOffset.asSeconds() + 2));
}

void Song::fastFourierTransform(ComplexArray& x)
{
    const size_t N = x.size();
    if( N <= 1)
    {
        return;
    }
    ComplexArray even = x[std::slice(0, N/2, 2)];
    ComplexArray  odd = x[std::slice(1, N/2, 2)];

    fastFourierTransform(even);
    fastFourierTransform(odd);

    for (size_t k = 0; k < N/2; ++k)
    {
        ComplexNumber t = std::polar(1.0, -2 * PI * k / N) * odd[k];

        x[k] = even[k] + t;
        x[k + N/2] = even[k] - t;
    }
}

void Song::update()
{
    sf::Time offset = sound.getPlayingOffset();
    unsigned sampleOffset = offset.asSeconds() * sampleRate * 2.f;
    maxAmplitude = 0;

    bool gotNewSampleThisFrame = false;
    for(unsigned i = lastSampleOffset + 1; i < sampleOffset; i++)
    {
        maxAmplitude = max(maxAmplitude, (int)abs(samples[sampleOffset]));
        if(!gotNewSampleThisFrame)
        {
            fftSamples[fftSampleIndex++] = samples[i];
            if(fftSampleIndex == FFT_SAMPLES)
            {
                gotNewSampleThisFrame = true;
                fftResult = ComplexArray(fftSamples, FFT_SAMPLES);
                Song::fastFourierTransform(fftResult);
                fftSampleIndex = 0;
            }
        }
    }
    lastSampleOffset = sampleOffset;

    if(loop)
    {
        if(sound.getStatus() == sf::Sound::Stopped)
        {
            sound.play();
        }
    }
}

int Song::getAmplitude()
{
    return maxAmplitude;
}

unsigned Song::getOffset()
{
    return lastSampleOffset;
}

int Song::getBeatMeasureSize(int beatRate)
{
    return (18900 * 2 * beatRate);
}

int Song::getBeatOffset(int beatRate)
{
    return lastSampleOffset % (18900 * 2 * beatRate);
}

int Song::beatScore(int beatRate)
{
    /// 3 = perfect
    /// 2 = good
    /// 1 = bad
    /// 0 = miss

    static const vector<unsigned> RANGES = {27000, 23000, 15000};

    for(int i = 2; i >= 0; i--)
    {

        if((lastSampleOffset + RANGES[i]/2) % (18900 * 2 * beatRate) < RANGES[i])
        {
            return i+1;
        }
    }
    return 0;
}

void Song::setVolume(float f)
{
    volume = f;
    sound.setVolume(f * baseVolume);
}

void Song::setLoop(bool b)
{
    loop = b;
}

bool Song::isFinished()
{
    return sound.getStatus() == sf::Sound::Stopped;
}
