#include "Sprite.h"

Sprite::Sprite(string textureName, int width, int height,
               int frameCount,
               int speed,
               float offsetX, float offsetY,
               bool loop):
    flipped(false),
    sprite(*ResourceManager::getTexture(textureName)),
    frameContainer(0, 0, width, height),
    w(width),
    h(height),
    frame(0),
    numFrames(frameCount),
    timer(0),
    speed(speed),
    resetDelay(0),
    angle(0),
    scaleX(1),
    scaleY(1),
    offset(offsetX, offsetY),
    reverse(false),
    frozen(false),
    loop(loop),
    done(false),
    rows(0),
    cols(0)
{

    if(w > (int)(sprite.getTexture()->getSize().x) || w <= 0)
    {
        this->w = sprite.getTexture()->getSize().x;
        frameContainer.width = this->w;
    }
    if( h > (int)(sprite.getTexture()->getSize().y) || h <= 0)
    {
        this->h = sprite.getTexture()->getSize().y;
        frameContainer.height = this->h;
    }

    sprite.setOrigin(this->w/2.f, this->h/2.f);
    sprite.setTextureRect(frameContainer);
    rows = sprite.getTexture()->getSize().y / this->h;
    cols = sprite.getTexture()->getSize().x / this->w;
    if(numFrames == -1)
    {
        numFrames = rows * cols;
    }
}
void Sprite::step(const sf::Vector2i& frameRectDelta, const sf::Vector2f& frameRectSizeMod)
{
    if(frozen)
    {
        return;
    }
    timer++;
    if(timer > speed)
    {
        if(frame == 0)
        {
            if(timer < speed + resetDelay)
            {
                return;
            }
        }
        if(reverse)
        {
            frame--;
        }
        else
        {
            frame++;
        }
        if(frame < 0)
        {
            if(loop)
            {
                frame = numFrames - 1;
            }
            else
            {
                frame = 0;
                done = true;
            }
        }

        if(frame >= numFrames)
        {
            if(loop)
            {
                frame = 0;
            }
            else
            {
                frame = numFrames-1;
                done = true;
            }
        }
        timer = 0;

        int x = (frame % cols) * w;
        int y = (int)(frame / cols) * h;
        sf::IntRect tRect = sf::IntRect(x + frameRectDelta.x, y + frameRectDelta.y,
                                        (int)(w*frameRectSizeMod.x), (int)(h*frameRectSizeMod.y));
        sprite.setTextureRect(tRect);
    }
    else if(frameRectDelta.x || frameRectDelta.y || frameRectSizeMod.x != 1 || frameRectSizeMod.y != 1)
    {
        int x = (frame % cols) * w;
        int y = (int)(frame / cols) * h;
        sprite.setTextureRect(sf::IntRect(x + frameRectDelta.x, y + frameRectDelta.y,
            (int)(w*frameRectSizeMod.x), (int)(h*frameRectSizeMod.y)));
    }
}
void Sprite::draw(sf::RenderTarget& window, sf::RenderStates states)
{
    if(flipped)
    {
        sprite.setScale(-1 * scaleX, scaleY);
    }
    else
    {
        sprite.setScale(scaleX, scaleY);
    }

    window.draw(sprite, states);
}

void Sprite::setFlipped(bool f)
{
    flipped = f;
}

void Sprite::setFrozen(bool f)
{
    frozen = f;
}

void Sprite::setReverse(bool r)
{
    if(reverse != r)
    {
        reverse = r;
        done = false;
    }
}

bool Sprite::getReverse()
{
    return reverse;
}
void Sprite::setOrigin(float x, float y)
{
    sprite.setOrigin(x, y);
}
void Sprite::setOffset(float x, float y)
{
    offset.x = x;
    offset.y = y;
}

void Sprite::setPosition(float x, float y)
{
    if(flipped)
        sprite.setPosition(x + offset.x, y-offset.y);
    else
        sprite.setPosition(x - offset.x, y-offset.y);
}

void Sprite::setPosition(const sf::Vector2f& pos)
{
    setPosition(pos.x, pos.y);
}
void Sprite::setPositionNoOffset(float x, float y)
{
    sprite.setPosition(x,y);
}
void Sprite::setRotation(float theta)
{
    angle = theta;
    sprite.setRotation(theta);
}
void Sprite::setFrame(int f)
{
    timer = 0;
    if(frame != f)
    {
        frame = f;
        frameContainer.left = (frame % cols) * w;
        frameContainer.top = (int)(frame / cols) * h;

        sprite.setTextureRect(frameContainer);
    }
}

void Sprite::setScale(float sx, float sy)
{
    scaleX = sx;
    scaleY = sy;
}

void Sprite::setScale(float s)
{
    setScale(s, s);
}

void Sprite::setColor(sf::Color color)
{
    sprite.setColor(color);
}
float Sprite::percentComplete()
{
    if(speed == 0)
    {
        return (frame+1) / (float)(numFrames);
    }
    return (frame * speed + timer) / (float)(numFrames * speed);
}
int Sprite::getFrame()
{
    return frame;
}
int Sprite::getTotalFrames()
{
    return numFrames * speed;
}
int Sprite::getNumFrames()
{
    return numFrames;
}
int Sprite::getTime()
{
    return timer;
}
int Sprite::getSpeed()
{
    return speed;
}
float Sprite::getRotation()
{
    return angle;
}
void Sprite::setSpeed(int s)
{
    speed = s;
}
sf::Vector2i Sprite::getSize()
{
    return sf::Vector2i(w, h);
}
sf::Vector2f Sprite::getPosition()
{
    return sprite.getPosition();
}
sf::Vector2f Sprite::getOrigin()
{
    return sprite.getOrigin();
}
void Sprite::reset()
{
    if(reverse)
    {
        frame = numFrames - 1;
    }
    else
    {
        frame = 0;
    }
    timer = 0;
    done = false;
    int x = (frame % cols) * w;
    int y = (int)(frame / cols) * h;
    sprite.setTextureRect(sf::IntRect(x, y, w, h));
}
bool Sprite::isFinished()
{
    return done;
}
void Sprite::setSmooth(bool val)
{
    //sprite.getTexture()->setSmooth(val);
}
void Sprite::rotate(float degrees)
{
    angle += degrees;
    sprite.setRotation(angle);
}
void Sprite::setDone(bool val)
{
    done = val;
}
